#version 330

uniform sampler2D DiffuseSampler;
uniform float Time;

varying vec2 texCoord;
varying vec2 oneTexel;

vec4 BlurColor (in vec2 Coord, in sampler2D Tex, in float MipBias)
{
    vec2 TexelSize = texCoord*oneTexel*vec2(4.);

    vec4  Color = texture(Tex, Coord, MipBias);
    Color += texture(Tex, Coord + vec2(TexelSize.x,0.0), MipBias);
    Color += texture(Tex, Coord + vec2(-TexelSize.x,0.0), MipBias);
    Color += texture(Tex, Coord + vec2(0.0,TexelSize.y), MipBias);
    Color += texture(Tex, Coord + vec2(0.0,-TexelSize.y), MipBias);
    Color += texture(Tex, Coord + vec2(TexelSize.x,TexelSize.y), MipBias);
    Color += texture(Tex, Coord + vec2(-TexelSize.x,TexelSize.y), MipBias);
    Color += texture(Tex, Coord + vec2(TexelSize.x,-TexelSize.y), MipBias);
    Color += texture(Tex, Coord + vec2(-TexelSize.x,-TexelSize.y), MipBias);

    return Color/8.0;
}

void main() {
    vec2 uv = texCoord;
    vec4 color = vec4(texture2D(DiffuseSampler, texCoord).rgb, 1.0);

    float x = (uv.x + 4.0 ) * (uv.y + 4.0 ) * (Time*32.);
    vec4 grain = vec4(mod((mod(x, 13.0) + 1.0) * (mod(x, 123.0) + 1.0), 0.01)-0.005) * 8.0;

    float Threshold = 0.3;
    float Intensity = 1.0;
    float BlurSize = 2.0;
    vec4 Highlight = clamp(BlurColor(uv, DiffuseSampler, BlurSize)-Threshold,0.0,1.0)*1.0/(1.0-Threshold);
    color.rgb = ((color.rgb - 0.5f) * 1.2 + 0.5f);
    color = 1.0-(1.0-color)*(1.0-Highlight*Intensity); //Screen Blend Mode
    
    color.b *= 1.2;
    color.r *= 0.9;

    gl_FragColor = color + grain;
}
